unit WinForm;

interface

uses
  System.Drawing, System.Collections, System.ComponentModel,
  System.Windows.Forms, System.Data,
  Microsoft.DirectX, Microsoft.DirectX.Direct3D,Microsoft.DirectX.Direct3DX;

type
  TWinForm = class(System.Windows.Forms.Form)
  {$REGION 'Designer Managed Code'}
  strict private
    /// <summary>
    /// Required designer variable.
    /// </summary>
    Components: System.ComponentModel.Container;
    /// <summary>
    /// Required method for Designer support - do not modify
    /// the contents of this method with the code editor.
    /// </summary>
    procedure InitializeComponent;
  {$ENDREGION}
  strict protected
    /// <summary>
    /// Clean up any resources being used.
    /// </summary>
    procedure Dispose(Disposing: Boolean); override;
  private
    { Private Declarations }
  public
    constructor Create;
  private
    urzadzenie : Device;
    wierzcholki: VertexBuffer;
  public
    function  InicjujGrafike: Boolean;
    procedure Rysuj;
  private
    function  UtworzBuforWierzch(var urzadzenie: Device): VertexBuffer;
    procedure WypelnijBuforWierzch(var bufor: VertexBuffer);
    procedure UstawMacierze;
  end;

  [assembly: RuntimeRequiredAttribute(TypeOf(TWinForm))]

implementation

{$AUTOBOX ON}

{$REGION 'Windows Form Designer generated code'}
/// <summary>
/// Required method for Designer support -- do not modify
/// the contents of this method with the code editor.
/// </summary>
procedure TWinForm.InitializeComponent;
begin
  // 
  // TWinForm
  // 
  Self.AutoScaleBaseSize := System.Drawing.Size.Create(5, 13);
  Self.ClientSize := System.Drawing.Size.Create(592, 566);
  Self.Name := 'TWinForm';
  Self.Text := 'DirectX.NET Przykad 5';
end;
{$ENDREGION}

procedure TWinForm.Dispose(Disposing: Boolean);
begin
  if Disposing then
  begin
    if Components <> nil then
      Components.Dispose();
  end;
  inherited Dispose(Disposing);
end;

constructor TWinForm.Create;
begin
  inherited Create;
  //
  // Required for Windows Form Designer support
  //
  InitializeComponent;
  //
  // TODO: Add any constructor code after InitializeComponent call
  //
end;

function TWinForm.InicjujGrafike: Boolean;
var
  parametry: PresentParameters;
begin
  try
    parametry:= PresentParameters.Create;
    parametry.Windowed:= true;
    parametry.SwapEffect:= Microsoft.DirectX.Direct3D.SwapEffect.Discard;
    urzadzenie:= Device.Create(0, DeviceType.Hardware, Self.Handle,
      CreateFlags.SoftwareVertexProcessing, [parametry]);
    wierzcholki:= UtworzBuforWierzch(urzadzenie);
    urzadzenie.RenderState.CullMode:= Cull.None;
    urzadzenie.RenderState.Lighting:= false;
    Result:= true;
  except
    on DirectXException do Result:=false;
  end;
end;

procedure TWinForm.Rysuj;
begin
  if (urzadzenie=nil) then Exit;

  urzadzenie.Clear(ClearFlags.Target, System.Drawing.Color.White, 1, 0);
  urzadzenie.BeginScene;

  UstawMacierze;
  urzadzenie.SetStreamSource(0, wierzcholki, 0);
  urzadzenie.DrawPrimitives(PrimitiveType.TriangleList, 0, 1);

  urzadzenie.EndScene;
  urzadzenie.Present;
end;

function TWinForm.UtworzBuforWierzch(var urzadzenie: Device): VertexBuffer;
var bufor: VertexBuffer;
begin
  urzadzenie.VertexFormat := CustomVertex.PositionColored.Format;
  bufor:= VertexBuffer.Create(typeof(CustomVertex.PositionColored),
3, urzadzenie, Usage.WriteOnly, CustomVertex.PositionColored.Format, Pool.Default);
  WypelnijBuforWierzch(bufor);
  Result:= bufor;
end;

procedure TWinForm.WypelnijBuforWierzch(var bufor: VertexBuffer);
var
   wierzch : Array[0..2] of CustomVertex.PositionColored;
   strumien: GraphicsStream;
begin
  strumien:=  bufor.Lock(0, 0, LockFlags.None);
  wierzch[0]:= CustomVertex.PositionColored.Create( 1, -0.5, 0, Color.Green.ToArgb());
  wierzch[1]:= CustomVertex.PositionColored.Create(-1, -0.5, 0, Color.Red.ToArgb());
  wierzch[2]:= CustomVertex.PositionColored.Create( 0,    1, 0, Color.Blue.ToArgb());

  strumien.Write(wierzch);
  bufor.Unlock;
end;

procedure TWinForm.UstawMacierze;
var angle: real;
begin
  angle:= Environment.TickCount / 1000.0;
  urzadzenie.Transform.World:= Matrix.RotationY(angle);

  urzadzenie.Transform.View:= Matrix.LookAtLH(Vector3.Create(0, 0, -3),
    Vector3.Create(0, 0, 0), Vector3.Create(0, 1, 0));
  urzadzenie.Transform.Projection :=
    Matrix.PerspectiveFovLH(Geometry.DegreeToRadian(60), 1.0, 1.0, 100.0);
end;

end.
